@@ -9,9 +9,10 @@ from .extractors.klustaextractors.klustaextractors import KlustaSortingExtractor
 from .extractors.kilosortextractors.kilosortextractors import KiloSortSortingExtractor, KiloSortRecordingExtractor
 from .extractors.numpyextractors.numpyextractors import NumpyRecordingExtractor, NumpySortingExtractor
 from .extractors.nwbextractors.nwbextractors import NwbRecordingExtractor, NwbSortingExtractor
-from .extractors.maxonerecordingextractor import MaxOneRecordingExtractor
-from .extractors.mea1krecordingextractor import Mea1kRecordingExtractor
-from .extractors.openephysextractors.openephysextractors import OpenEphysRecordingExtractor, OpenEphysSortingExtractor
+from .extractors.openephysextractors.openephysextractors import OpenEphysRecordingExtractor, \
+    OpenEphysSortingExtractor, OpenEphysNPIXRecordingExtractor
+from .extractors.maxwellextractors import MaxOneRecordingExtractor, MaxOneSortingExtractor, MaxTwoRecordingExtractor, \
+    MaxTwoSortingExtractor
 from .extractors.phyextractors.phyextractors import PhyRecordingExtractor, PhySortingExtractor
 from .extractors.bindatrecordingextractor.bindatrecordingextractor import BinDatRecordingExtractor
 from .extractors.spykingcircusextractors.spykingcircusextractors import SpykingCircusSortingExtractor, \
@@ -23,9 +24,18 @@ from .extractors.mcsh5recordingextractor.mcsh5recordingextractor import MCSH5Rec
 from .extractors.shybridextractors import SHYBRIDRecordingExtractor, SHYBRIDSortingExtractor
 from .extractors.nixioextractors.nixioextractors import NIXIORecordingExtractor, NIXIOSortingExtractor
 from .extractors.neoextractors import (PlexonRecordingExtractor, PlexonSortingExtractor,
-                                       NeuralynxRecordingExtractor, NeuralynxSortingExtractor, MCSRawRecordingExtractor)
-from .extractors.neuroscopeextractors import NeuroscopeRecordingExtractor,NeuroscopeSortingExtractor,NeuroscopeMultiSortingExtractor
+                                       NeuralynxRecordingExtractor, NeuralynxSortingExtractor,
+                                       BlackrockRecordingExtractor, BlackrockSortingExtractor,
+                                       MCSRawRecordingExtractor)
+from .extractors.neuroscopeextractors import NeuroscopeRecordingExtractor, NeuroscopeMultiRecordingTimeExtractor, \
+    NeuroscopeSortingExtractor, NeuroscopeMultiSortingExtractor
 from .extractors.waveclussortingextractor import WaveClusSortingExtractor
+from .extractors.yassextractors import YassSortingExtractor
+from .extractors.combinatosortingextractor import CombinatoSortingExtractor
+from .extractors.alfsortingextractor import ALFSortingExtractor
+from .extractors.cedextractors import CEDRecordingExtractor
+from .extractors.cellexplorersortingextractor import CellExplorerSortingExtractor
+from .extractors.neuropixelsdatrecordingextractor import NeuropixelsDatRecordingExtractor
 
 recording_extractor_full_list = [
     MdaRecordingExtractor,
@@ -33,6 +43,7 @@ recording_extractor_full_list = [
     BiocamRecordingExtractor,
     ExdirRecordingExtractor,
     OpenEphysRecordingExtractor,
+    OpenEphysNPIXRecordingExtractor,
     IntanRecordingExtractor,
     BinDatRecordingExtractor,
     KlustaRecordingExtractor,
@@ -41,20 +52,25 @@ recording_extractor_full_list = [
     SpikeGLXRecordingExtractor,
     PhyRecordingExtractor,
     MaxOneRecordingExtractor,
-    Mea1kRecordingExtractor,
+    MaxTwoRecordingExtractor,
     MCSH5RecordingExtractor,
-    MCSRawRecordingExtractor,
     SHYBRIDRecordingExtractor,
     NIXIORecordingExtractor,
+    NwbRecordingExtractor,
     NeuroscopeRecordingExtractor,
+    NeuroscopeMultiRecordingTimeExtractor,
+    CEDRecordingExtractor,
+    NeuropixelsDatRecordingExtractor,
 
     # neo based
     PlexonRecordingExtractor,
-    NeuralynxRecordingExtractor
-    
+    NeuralynxRecordingExtractor,
+    BlackrockRecordingExtractor,
+    MCSRawRecordingExtractor,
 ]
 
-recording_extractor_dict = {recording_class.extractor_name: recording_class for recording_class in recording_extractor_full_list}
+recording_extractor_dict = {recording_class.extractor_name: recording_class
+                            for recording_class in recording_extractor_full_list}
 installed_recording_extractor_list = [rx for rx in recording_extractor_full_list if rx.installed]
 
 sorting_extractor_full_list = [
@@ -69,20 +85,28 @@ sorting_extractor_full_list = [
     PhySortingExtractor,
     SpykingCircusSortingExtractor,
     TridesclousSortingExtractor,
+    MaxTwoSortingExtractor,
+    MaxOneSortingExtractor,
     NpzSortingExtractor,
     SHYBRIDSortingExtractor,
     NIXIOSortingExtractor,
     NeuroscopeSortingExtractor,
     NeuroscopeMultiSortingExtractor,
+    NwbSortingExtractor,
     WaveClusSortingExtractor,
+    YassSortingExtractor,
+    CombinatoSortingExtractor,
+    ALFSortingExtractor,
     # neo based
     PlexonSortingExtractor,
     NeuralynxSortingExtractor,
-    
+    BlackrockSortingExtractor,
+    CellExplorerSortingExtractor
 ]
 
 installed_sorting_extractor_list = [sx for sx in sorting_extractor_full_list if sx.installed]
 sorting_extractor_dict = {sorting_class.extractor_name: sorting_class for sorting_class in sorting_extractor_full_list}
 
 writable_sorting_extractor_list = [sx for sx in installed_sorting_extractor_list if sx.is_writable]
-writable_sorting_extractor_dict = {sorting_class.extractor_name: sorting_class for sorting_class in writable_sorting_extractor_list}
+writable_sorting_extractor_dict = {sorting_class.extractor_name: sorting_class
+                                   for sorting_class in writable_sorting_extractor_list}
