@@ -1,18 +1,19 @@
 from spikeextractors import RecordingExtractor
 from spikeextractors import SortingExtractor
-from spikeextractors.extraction_tools import write_to_binary_dat_format, check_get_traces_args, check_valid_unit_id
+from spikeextractors.extraction_tools import write_to_binary_dat_format, check_get_traces_args, \
+    check_get_unit_spike_train
 
 import json
 import numpy as np
 from pathlib import Path
 from .mdaio import DiskReadMda, readmda, writemda64, MdaHeader
-import os
 import shutil
 
 
 class MdaRecordingExtractor(RecordingExtractor):
-    extractor_name = 'MdaRecordingExtractor'
+    extractor_name = 'MdaRecording'
     has_default_locations = True
+    has_unscaled = False
     installed = True  # check at class level if installed or not
     is_writable = True
     mode = 'folder'
@@ -24,10 +25,10 @@ class MdaRecordingExtractor(RecordingExtractor):
         timeseries0 = dataset_directory / raw_fname
         self._dataset_params = read_dataset_params(dataset_directory, params_fname)
         self._sampling_frequency = self._dataset_params['samplerate'] * 1.0
-        self._timeseries_path = os.path.abspath(timeseries0)
+        self._timeseries_path = str(timeseries0.absolute())
         geom0 = dataset_directory / geom_fname
         self._geom_fname = geom0
-        self._geom = np.loadtxt(self._geom_fname, delimiter=',',ndmin=2)
+        self._geom = np.loadtxt(self._geom_fname, delimiter=',', ndmin=2)
         X = DiskReadMda(self._timeseries_path)
         if self._geom.shape[0] != X.N1():
             raise Exception(
@@ -49,14 +50,15 @@ class MdaRecordingExtractor(RecordingExtractor):
         return self._sampling_frequency
 
     @check_get_traces_args
-    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None):
+    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None, return_scaled=True):
         X = DiskReadMda(self._timeseries_path)
         recordings = X.readChunk(i1=0, i2=start_frame, N1=X.N1(), N2=end_frame - start_frame)
         recordings = recordings[channel_ids, :]
         return recordings
 
-    def write_to_binary_dat_format(self, save_path, time_axis=0, dtype=None, chunk_size=None, chunk_mb=500):
-        '''Saves the traces of this recording extractor into binary .dat format.
+    def write_to_binary_dat_format(self, save_path, time_axis=0, dtype=None, chunk_size=None, chunk_mb=500,
+                                   n_jobs=1, joblib_backend='loky', verbose=False):
+        """Saves the traces of this recording extractor into binary .dat format.
 
         Parameters
         ----------
@@ -68,12 +70,17 @@ class MdaRecordingExtractor(RecordingExtractor):
         dtype: dtype
             Type of the saved data. Default float32
         chunk_size: None or int
-            If not None then the file is saved in chunks.
-            This avoid to much memory consumption for big files.
-            If 'auto' the file is saved in chunks of ~ 500Mb
+            Size of each chunk in number of frames.
+            If None (default) and 'chunk_mb' is given, the file is saved in chunks of 'chunk_mb' Mb (default 500Mb)
         chunk_mb: None or int
             Chunk size in Mb (default 500Mb)
-        '''
+        n_jobs: int
+            Number of jobs to use (Default 1)
+        joblib_backend: str
+            Joblib backend for parallel processing ('loky', 'threading', 'multiprocessing')
+        verbose: bool
+            If True, output is verbose
+        """
         X = DiskReadMda(self._timeseries_path)
         header_size = X._header.header_size
         if dtype is None or dtype == self.get_dtype():
@@ -85,15 +92,18 @@ class MdaRecordingExtractor(RecordingExtractor):
                 print('Error occurred while copying:', e)
                 print('Writing to binary')
                 write_to_binary_dat_format(self, save_path=save_path, time_axis=time_axis, dtype=dtype,
-                                           chunk_size=chunk_size, chunk_mb=chunk_mb)
+                                           chunk_size=chunk_size, chunk_mb=chunk_mb, n_jobs=n_jobs,
+                                           joblib_backend=joblib_backend, verbose=verbose)
         else:
             write_to_binary_dat_format(self, save_path=save_path, time_axis=time_axis, dtype=dtype,
-                                       chunk_size=chunk_size, chunk_mb=chunk_mb)
+                                       chunk_size=chunk_size, chunk_mb=chunk_mb, n_jobs=n_jobs,
+                                       joblib_backend=joblib_backend, verbose=verbose)
 
     @staticmethod
     def write_recording(recording, save_path, params=dict(), raw_fname='raw.mda', params_fname='params.json',
-                        geom_fname='geom.csv', dtype=None, chunk_size=None, chunk_mb=500):
-        '''
+                        geom_fname='geom.csv', dtype=None, chunk_size=None, n_jobs=None, chunk_mb=500, verbose=False):
+        """
+        Writes recording to file in MDA format.
 
         Parameters
         ----------
@@ -112,15 +122,17 @@ class MdaRecordingExtractor(RecordingExtractor):
         dtype: dtype
             dtype to be used. If None dtype is same as recording traces.
         chunk_size: None or int
-            Number of chunks to save the file in. This avoid to much memory consumption for big files.
-            If None and 'chunk_mb' is given, the file is saved in chunks of 'chunk_mb' Mb (default 500Mb)
+            Size of each chunk in number of frames.
+            If None (default) and 'chunk_mb' is given, the file is saved in chunks of 'chunk_mb' Mb (default 500Mb)
+        n_jobs: int
+            Number of jobs to use (Default 1)
         chunk_mb: None or int
             Chunk size in Mb (default 500Mb)
-        '''
+        verbose: bool
+            If True, output is verbose
+        """
         save_path = Path(save_path)
-        if not save_path.exists():
-            if not save_path.is_dir():
-                os.makedirs(str(save_path))
+        save_path.mkdir(parents=True, exist_ok=True)
         save_file_path = save_path / raw_fname
         parent_dir = save_path
         channel_ids = recording.get_channel_ids()
@@ -129,9 +141,6 @@ class MdaRecordingExtractor(RecordingExtractor):
 
         geom = recording.get_channel_locations()
 
-        if not save_path.is_dir():
-            os.mkdir(save_path)
-
         if dtype is None:
             dtype = recording.get_dtype()
 
@@ -144,8 +153,8 @@ class MdaRecordingExtractor(RecordingExtractor):
             header = MdaHeader(dt0=dtype, dims0=(num_chan, num_frames))
             header.write(f)
             # takes care of the chunking
-            write_to_binary_dat_format(recording, file_handle=f, dtype=dtype, chunk_size=chunk_size,
-                                       chunk_mb=chunk_mb)
+            write_to_binary_dat_format(recording, file_handle=f, dtype=dtype, n_jobs=n_jobs, chunk_size=chunk_size,
+                                       chunk_mb=chunk_mb, verbose=verbose)
 
         params["samplerate"] = recording.get_sampling_frequency()
         with (parent_dir / params_fname).open('w') as f:
@@ -154,7 +163,7 @@ class MdaRecordingExtractor(RecordingExtractor):
 
 
 class MdaSortingExtractor(SortingExtractor):
-    extractor_name = 'MdaSortingExtractor'
+    extractor_name = 'MdaSorting'
     installed = True  # check at class level if installed or not
     is_writable = True
     mode = 'file'
@@ -166,28 +175,21 @@ class MdaSortingExtractor(SortingExtractor):
         self._firings_path = file_path
         self._firings = readmda(self._firings_path)
         self._max_channels = self._firings[0, :]
-        self._times = self._firings[1, :]
+        self._spike_times = self._firings[1, :]
         self._labels = self._firings[2, :]
         self._unit_ids = np.unique(self._labels).astype(int)
         self._sampling_frequency = sampling_frequency
-        for unit_id in self._unit_ids:
-            inds = np.where(self._labels == unit_id)
-            max_channels = self._max_channels[inds].astype(int)
-            self.set_unit_property(unit_id, 'mda_max_channel', max_channels[0])
         self._kwargs = {'file_path': str(Path(file_path).absolute()), 'sampling_frequency': sampling_frequency}
 
     def get_unit_ids(self):
         return list(self._unit_ids)
 
-    @check_valid_unit_id
+    @check_get_unit_spike_train
     def get_unit_spike_train(self, unit_id, start_frame=None, end_frame=None):
-        start_frame, end_frame = self._cast_start_end_frame(start_frame, end_frame)
-        if start_frame is None:
-            start_frame = 0
-        if end_frame is None:
-            end_frame = np.Inf
-        inds = np.where((self._labels == unit_id) & (start_frame <= self._times) & (self._times < end_frame))
-        return np.rint(self._times[inds]).astype(int)
+
+        inds = np.where(
+            (self._labels == unit_id) & (start_frame <= self._spike_times) & (self._spike_times < end_frame))
+        return np.rint(self._spike_times[inds]).astype(int)
 
     @staticmethod
     def write_sorting(sorting, save_path, write_primary_channels=False):
@@ -201,7 +203,7 @@ class MdaSortingExtractor(SortingExtractor):
             labels_list.append(np.ones(times.shape) * unit_id)
             if write_primary_channels:
                 if 'max_channel' in sorting.get_unit_property_names(unit_id):
-                    primary_channels_list.append([sorting.get_unit_property(unit_id, 'max_channel')]*times.shape[0])
+                    primary_channels_list.append([sorting.get_unit_property(unit_id, 'max_channel')] * times.shape[0])
                 else:
                     raise ValueError(
                         "Unable to write primary channels because 'max_channel' spike feature not set in unit " + str(
@@ -232,7 +234,7 @@ def _concatenate(list):
 
 def read_dataset_params(dsdir, params_fname):
     fname1 = dsdir / params_fname
-    if not os.path.exists(fname1):
+    if not fname1.is_file():
         raise Exception('Dataset parameter file does not exist: ' + fname1)
     with open(fname1) as f:
         return json.load(f)
