@@ -1,28 +1,35 @@
 from spikeextractors import RecordingExtractor
 from spikeextractors import SortingExtractor
-from spikeextractors.extraction_tools import check_get_traces_args, check_valid_unit_id
+from spikeextractors.extraction_tools import check_get_traces_args, check_get_unit_spike_train
 
 import numpy as np
 from pathlib import Path
+from distutils.version import StrictVersion
 
 try:
     import MEArec as mr
     import neo
     import quantities as pq
-    HAVE_MREX = True
+    if StrictVersion(mr.__version__) >= '1.5.0':
+        HAVE_MREX = True
+    else:
+        print("MEArec version requires an update (>=1.5). Please upgrade with 'pip install --upgrade MEArec'")
+        HAVE_MREX = False
 except ImportError:
     HAVE_MREX = False
 
 
 class MEArecRecordingExtractor(RecordingExtractor):
-    extractor_name = 'MEArecRecordingExtractor'
+    extractor_name = 'MEArecRecording'
     has_default_locations = True
+    has_unscaled = False
     installed = HAVE_MREX  # check at class level if installed or not
     is_writable = True
     mode = 'file'
     installation_mesg = "To use the MEArec extractors, install MEArec: \n\n pip install MEArec\n\n"  # error message when not installed
 
     def __init__(self, file_path, locs_2d=True):
+        assert self.installed, self.installed
         self._recording_path = file_path
         self._fs = None
         self._positions = None
@@ -39,12 +46,11 @@ class MEArecRecordingExtractor(RecordingExtractor):
         self._kwargs = {'file_path': str(Path(file_path).absolute()), 'locs_2d': locs_2d}
 
     def _initialize(self):
-        assert HAVE_MREX, self.installation_mesg
         self._recgen = mr.load_recordings(recordings=self._recording_path, return_h5_objects=True, check_suffix=False,
                                           load=['recordings', 'channel_positions'])
         self._fs = self._recgen.info['recordings']['fs']
         self._recordings = self._recgen.recordings
-        self._num_channels, self._num_frames = self._recordings.shape
+        self._num_frames, self._num_channels = self._recordings.shape
         if len(np.array(self._recgen.channel_positions)) == self._num_channels:
             self._locations = np.array(self._recgen.channel_positions)
             if self._locs_2d:
@@ -70,19 +76,20 @@ class MEArecRecordingExtractor(RecordingExtractor):
         return self._fs
 
     @check_get_traces_args
-    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None):
+    def get_traces(self, channel_ids=None, start_frame=None, end_frame=None, return_scaled=True):
         if np.any(np.diff(channel_ids) < 0):
-            sorted_idx = np.argsort(channel_ids)
-            recordings = self._recordings[np.sort(channel_ids), start_frame:end_frame]
-            return recordings[sorted_idx]
+            sorted_channel_ids = np.sort(channel_ids)
+            sorted_idx = np.array([list(sorted_channel_ids).index(ch) for ch in channel_ids])
+            recordings = self._recordings[start_frame:end_frame, sorted_channel_ids]
+            return np.array(recordings[:, sorted_idx]).T
         else:
             if sorted(channel_ids) == channel_ids and np.all(np.diff(channel_ids) == 1):
                 channel_ids = slice(channel_ids[0], channel_ids[0] + len(channel_ids))
-            return self._recordings[channel_ids, start_frame:end_frame]
+            return np.array(self._recordings[start_frame:end_frame, channel_ids]).T
         
     @staticmethod
     def write_recording(recording, save_path, check_suffix=True):
-        '''
+        """
         Save recording extractor to MEArec format.
         Parameters
         ----------
@@ -90,7 +97,7 @@ class MEArecRecordingExtractor(RecordingExtractor):
             Recording extractor object to be saved
         save_path: str
             .h5 or .hdf5 path
-        '''
+        """
         assert HAVE_MREX, MEArecRecordingExtractor.installation_mesg
         save_path = Path(save_path)
         if save_path.is_dir():
@@ -98,7 +105,7 @@ class MEArecRecordingExtractor(RecordingExtractor):
             save_path = save_path / 'recording.h5'
         if (save_path.suffix == '.h5' or save_path.suffix == '.hdf5') or (not check_suffix):
             info = {'recordings': {'fs': recording.get_sampling_frequency()}}
-            rec_dict = {'recordings': recording.get_traces()}
+            rec_dict = {'recordings': recording.get_traces().transpose()}
             if 'location' in recording.get_shared_channel_property_names():
                 positions = recording.get_channel_locations()
                 rec_dict['channel_positions'] = positions
@@ -109,13 +116,14 @@ class MEArecRecordingExtractor(RecordingExtractor):
 
 
 class MEArecSortingExtractor(SortingExtractor):
-    extractor_name = 'MEArecSortingExtractor'
+    extractor_name = 'MEArecSorting'
     installed = HAVE_MREX  # check at class level if installed or not
     is_writable = True
     mode = 'file'
     installation_mesg = "To use the MEArec extractors, install MEArec: \n\n pip install MEArec\n\n"  # error message when not installed
 
     def __init__(self, file_path):
+        assert self.installed, self.installed
         SortingExtractor.__init__(self)
         self._recording_path = file_path
         self._num_units = None
@@ -126,7 +134,6 @@ class MEArecSortingExtractor(SortingExtractor):
         self._kwargs = {'file_path': str(Path(file_path).absolute())}
 
     def _initialize(self):
-        assert HAVE_MREX, "To use the MEArec extractors, install MEArec: \n\n pip install MEArec\n\n"
         recgen = mr.load_recordings(recordings=self._recording_path, return_h5_objects=True, check_suffix=False,
                                     load=['spiketrains'])
         self._num_units = len(recgen.spiketrains)
@@ -152,13 +159,8 @@ class MEArecSortingExtractor(SortingExtractor):
             self._initialize()
         return self._num_units
 
-    @check_valid_unit_id
+    @check_get_unit_spike_train
     def get_unit_spike_train(self, unit_id, start_frame=None, end_frame=None):
-        start_frame, end_frame = self._cast_start_end_frame(start_frame, end_frame)
-        if start_frame is None:
-            start_frame = 0
-        if end_frame is None:
-            end_frame = np.Inf
         if self._spike_trains is None:
             self._initialize()
         times = (self._spike_trains[self.get_unit_ids().index(unit_id)].times.rescale('s') *
@@ -168,7 +170,7 @@ class MEArecSortingExtractor(SortingExtractor):
 
     @staticmethod
     def write_sorting(sorting, save_path, sampling_frequency, check_suffix=True):
-        '''
+        """
         Save sorting extractor to MEArec format.
         Parameters
         ----------
@@ -179,7 +181,7 @@ class MEArecSortingExtractor(SortingExtractor):
         sampling_frequency: int
             Sampling frequency in Hz
 
-        '''
+        """
         assert HAVE_MREX, MEArecSortingExtractor.installation_mesg
         save_path = Path(save_path)
         if save_path.is_dir():
