@@ -1,12 +1,12 @@
 from spikeextractors import SortingExtractor
 from pathlib import Path
-from spikeextractors.extraction_tools import check_valid_unit_id
+from spikeextractors.extraction_tools import check_get_unit_spike_train
 import numpy as np
 
 
 class NpzSortingExtractor(SortingExtractor):
     """
-    Dead simple format super light base on the NPZ numpy format.
+    Dead simple and super light format based on the NPZ numpy format.
     https://docs.scipy.org/doc/numpy/reference/generated/numpy.savez.html#numpy.savez
 
     It is in fact an arichive of several .npy format.
@@ -14,7 +14,7 @@ class NpzSortingExtractor(SortingExtractor):
 
 
     """
-    extractor_name = 'NpzSortingExtractor'
+    extractor_name = 'NpzSorting'
     installed = True # depend only on numpy
     installation_mesg = "Always installed"
     is_writable = True
@@ -39,9 +39,8 @@ class NpzSortingExtractor(SortingExtractor):
     def get_unit_ids(self):
         return list(self.unit_ids)
 
-    @check_valid_unit_id
+    @check_get_unit_spike_train
     def get_unit_spike_train(self, unit_id, start_frame=None, end_frame=None):
-        start_frame, end_frame = self._cast_start_end_frame(start_frame, end_frame)
         spike_times = self.spike_indexes[self.spike_labels == unit_id]
         if start_frame is not None:
             spike_times = spike_times[spike_times >= start_frame]
